<?php
/*
* A payment plugin called "v12". This is the main file of the plugin.
*/

// You need to extend from the hikashopPaymentPlugin class which already define lots of functions in order to simplify your work
class plgHikashoppaymentV12 extends hikashopPaymentPlugin
{
	//List of the plugin's accepted currencies. The plugin won't appear on the checkout if the current currency is not in that list. You can remove that attribute if you want your payment plugin to display for all the currencies
	var $accepted_currencies = array(
		'GBP','USD','EUR'
	);

	// Multiple plugin configurations. It should usually be set to true
	var $multiple = true;

	//Payment plugin name (the name of the PHP file)
	var $name = 'v12';

	// This array contains the specific configuration needed (Back end > payment plugin edition), depending of the plugin requirements.
	// They will vary based on your needs for the integration with your payment gateway.
	// The first parameter is the name of the field. In upper case for a translation key.
	// The available types (second parameter) are: input (an input field), html (when you want to display some custom HTML to the shop owner), textarea (when you want the shop owner to write a bit more than in an input field), big-textarea (when you want the shop owner to write a lot more than in an input field), boolean (for a yes/no choice), checkbox (for checkbox selection), list (for dropdown selection) , orderstatus (to be able to select between the available order statuses)
	// The third parameter is the default value.
	var $pluginConfig = array(
		// User's identifier on the payment platform
		'retailer_id' => array("Retailer ID",'input'),
		'retailer_guid' => array("Retailer GUID",'input'),
		'retailer_test_id' => array("Retailer Testing ID",'input'),
		'retailer_test_guid' => array("Retailer Testing GUID",'input'),
		'authentication_key' => array("Authentication Key",'input'),
		// Option to send the user an email to the application form
		'vlink' => array('Send application link email', 'boolean','1'),
		// To allow (or not) notifications from the payment platform. The plugin can only work if notifications are allowed
		'notification' => array('ALLOW_NOTIFICATIONS_FROM_X', 'boolean','0'),
		// Write some things on the debug file
		'testmode' => array('Test Mode', 'boolean','0'),
		'debug' => array('DEBUG', 'boolean','0'),
		// The URL where the user is redirected after a fail during the payment process
		'cancel_url' => array('CANCEL_URL_DEFINE','html',''),
		// The URL where the user is redirected after the payment is done on the payment gateway. It's a pre determined URL that has to be given to the payment gateway
		'return_url_gateway' => array('RETURN_URL_DEFINE', 'html',''),
		// The URL where the user is redirected by HikaShop after the payment is done ; "Thank you for purchase" page
		'return_url' => array('RETURN_URL', 'input'),
		// The URL where the payment platform the user about the payment (fail or success)
		'notify_url' => array('NOTIFY_URL_DEFINE','html',''),
		// Invalid status for order in case of problem during the payment process
		'invalid_status' => array('INVALID_STATUS', 'orderstatus'),
		// Valid status for order if the payment has been done well
		'confirmed_status' => array('Confirmed Status', 'orderstatus')
	);

	/**
	* The constructor is optional if you don't need to initialize some parameters of some fields of the configuration and not that it can also be done in the getPaymentDefaultValues function as you will see later on
	*/
	function __construct(&$subject, $config)
	{
		$this->pluginConfig['notification'][0] =  JText::sprintf('ALLOW_NOTIFICATIONS_FROM_X','V12');

		// This is the cancel URL of HikaShop that should be given to the payment gateway so that it can redirect to it when the user cancel the payment on the payment gateway page. That URL will automatically cancel the order of the user and redirect him to the checkout so that he can choose another payment method
		$this->pluginConfig['cancel_url'][2] = HIKASHOP_LIVE."index.php?option=com_hikashop&ctrl=order&task=cancel_order";

		// This is the "thank you" or "return" URL of HikaShop that should be given to the payment gateway so that it can redirect to it when the payment of the user is valid. That URL will reinit some variables in the session like the cart and will then automatically redirect to the "return_url" parameter
		$this->pluginConfig['return_url'][2] = HIKASHOP_LIVE."index.php?option=com_hikashop&ctrl=checkout&task=after_end";

		// This is the "notification" URL of HikaShop that should be given to the payment gateway so that it can send a request to that URL in order to tell HikaShop that the payment has been done (sometimes the payment gateway doesn't do that and passes the information to the return URL, in which case you need to use that notification URL as return URL and redirect the user to the HikaShop return URL at the end of the onPaymentNotification function)
		$this->pluginConfig['notify_url'][2] = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=notify&amp;notif_payment='.$this->name.'&tmpl=component';

		return parent::__construct($subject, $config);
	}


	/**
	* This function is called at the end of the checkout. That's the function which should display your payment gateway redirection form with the data from HikaShop
	*/
	function onAfterOrderConfirm(&$order, &$methods, $method_id)
	{
		// This is a mandatory line in order to initialize the attributes of the payment method
		parent::onAfterOrderConfirm($order,$methods,$method_id);

		// Here we can do some checks on the options of the payment method and make sure that every required parameter is set and otherwise display an error message to the user
		// The plugin can only work if those parameters are configured on the website's backend
		if ($this->payment_params->testmode == false)
		{
			if (empty($this->payment_params->retailer_id))
			{
				$this->app->enqueueMessage('You have to configure a retailer id for the V12 plugin payment first : check your plugin\'s parameters, on your website backend', 'error');
				return false;
			}
			else
			{
				$retailer_id = $this->payment_params->retailer_id;
			}

			if (empty($this->payment_params->retailer_guid))
			{
				$this->app->enqueueMessage('You have to configure a retailer guid for the V12 plugin payment first : check your plugin\'s parameters, on your website backend', 'error');
				return false;
			}
			else
			{
				$retailer_guid = $this->payment_params->retailer_guid;
			}
		}
		else
		{
			//testmode is on
			if (empty($this->payment_params->retailer_test_id))
			{
				$this->app->enqueueMessage('You have to configure a retailer test id for the V12 plugin payment first : check your plugin\'s parameters, on your website backend', 'error');
				return false;
			}
			else
			{
				$retailer_id = $this->payment_params->retailer_test_id;
			}

			if (empty($this->payment_params->retailer_test_guid))
			{
				$this->app->enqueueMessage('You have to configure a retailer test guid for the V12 plugin payment first : check your plugin\'s parameters, on your website backend', 'error');
				return false;
			}
			else
			{
				$retailer_guid = $this->payment_params->retailer_test_guid;
			}
		}

		if (empty($this->payment_params->authentication_key))
		{
			$this->app->enqueueMessage('You have to configure an authenticationkey for the V12 plugin payment first : check your plugin\'s parameters, on your website backend', 'error');
			return false;
		}
		else
		{
			// Here, all the required parameters are valid, so we can proceed to the payment platform

			// The order's amount, here in cents and rounded with 2 decimals because of the payment platform's requirements
			// There is a lot of information in the $order variable, such as price with/without taxes, customer info, products... you can do a var_dump here if you need to display all the available information

			$amount = round($order->cart->full_total->prices[0]->price_value_with_tax, 2);

			$vars = array();

			$vars['Customer'] = array(
				'EmailAddress' => $this->user->user_email,
				'FirstName' => $order->cart->billing_address->address_firstname,
				'LastName' => $order->cart->billing_address->address_lastname,
			);

			$vars['Retailer'] = array(
				'AuthenticationKey' => $this->payment_params->authentication_key,
				'RetailerGuid' => $retailer_guid,
				'RetailerId' => $retailer_id,
			);

			/*Duplicate sales reference values
			eDuplicateSalesReferenceMethod
			0 ShowError //Always show an error
			1 ShowOriginalApplication //If all aplication details are the same, show the original form
			2 CreateNewApplication //create a new application with the same sales id

			We use ShowOriginalApplication incase there are any issues with the ajax or a user clicks a button twice (We prevent this anyway with js)
			*/

			$vars['Order'] = array(
				'CashPrice' => $amount,
				'vLink' => $this->payment_params->vlink,
				'DuplicateSalesReferenceMethod' => 'ShowOriginalApplication',
				'SalesReference' => $order->cart->order_id,
			);

			$vars['Lines'] = array();
			foreach ($order->cart->products as $product) {
				$vars['Lines'][] = $product;
			}

			$this->total = $amount;
			$this->data = rawurlencode(json_encode($vars));

			// Ending the checkout, ready to be redirect to the plateform payment final form
			// The showPage function will call the example_end.php file which will display the redirection form containing all the parameters for the payment platform

			return $this->showPage('end');
		}
	}

	/**
	* To set the specific configuration (back end) default values (see $pluginConfig array)
	*/
	function getPaymentDefaultValues(&$element)
	{
		$element->payment_name = 'V12';
		$element->payment_description = 'You can apply for finance using this method';
		$element->payment_images = '';
		$element->payment_params->address_type = 'billing';
		$element->payment_params->notification = 1;
		$element->payment_params->invalid_status = 'cancelled';
		$element->payment_params->confirmed_status = 'confirmed';
	}

	/**
	* After submiting the platform payment form, this is where the website will receive the response information from the payment gateway servers and then validate or not the order
	*/
	function onPaymentNotification(&$statuses)
	{
		// We first create a filtered array from the parameters received
		$vars = array();

		$filter = JFilterInput::getInstance();

		// A loop to create an array $var with all the parameters sent by the payment gateway with a POST method, and loaded in the $_REQUEST
		foreach($_REQUEST as $key => $value)
		{
			$key = $filter->clean($key);
			$value = JRequest::getString($key);
			$vars[$key] = $value;
		}

		// The load the parameters of the plugin in $this->payment_params and the order data based on the order_id coming from the payment platform
		$order_id = (int)@$vars['SalesReference'];
		$dbOrder = $this->getOrder($order_id);

		// With the order, we can load the payment method, and thus all the payment parameters
		$this->loadPaymentParams($dbOrder);
		if(empty($this->payment_params))
		return false;
		$this->loadOrderData($dbOrder);

		// Here we are configuring the "succes URL" and the "fail URL". After checking all the parameters sent by the payment gateway, we will redirect the customer to one or another of those URL (not necessary for our example platform).
		$return_url = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=after_end&order_id='.$order_id.$this->url_itemid;
		$cancel_url = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=order&task=cancel_order&order_id='.$order_id.$this->url_itemid;

		// Debug mode activated or not
		if($this->payment_params->debug)
		{
			// Here we display debug information which will be catched by HikaShop and stored in the payment log file available in the configuration's Files section.
			echo print_r($vars,true)."\n\n\n";
			echo print_r($dbOrder,true)."\n\n\n";
		}

		//The other info sent from v12
		$ApplicationId = JFilterInput::getInstance()->clean($vars['ApplicationId'], 'STRING');
		$Status = JFilterInput::getInstance()->clean($vars['Status'], 'INT');

		//These are the possible status notifications
		/*
		0 Error
		1 Acknowledged
		2 Referred
		3 Declined
		4 Accepted
		5 AwaitingFulfilment
		6 PaymentRequested
		7 PaymentProcessed
		100 Cancelled
		*/

		// The payment platform returns a code, corresponding to the state of the operation. Here, the "success" code is 0000. It means that any other code correspond to a payment failure > process aborted
		if ($Status == '5')
		{
			$this->modifyOrder($order_id, $this->payment_params->confirmed_status, true, true);

			return true;
		}
		elseif ($Status == '0' || $Status == '3' || $Status == '100') {
			// Here we display debug information which will be catched by HikaShop and stored in the payment log file available in the configuration's Files section.
			if($this->payment_params->debug)
			echo 'order '. $order_id . ' status' . $Status ."\n\n\n";

			// This function modifies the order with the id $order_id, to attribute it the status invalid_status.
			$this->modifyOrder($order_id, $this->payment_params->invalid_status, true, true);

			//To redirect the user, if needed. Here the redirection is useless : we are on server side (and not user side, so the redirect won't work), and the cancel url has been set on the payment platform merchant account
			// $this->app->redirect($cancel_url);
			return false;
		}
	}

}
